@@ -6,6 +6,8 @@
 
 # base classes for model objects
 
+from warpseq.utils.serialization import save_object, load_object
+
 COUNTER = 0
 
 class BaseObject(object):
@@ -16,7 +18,7 @@ class BaseObject(object):
         length = len(alist)
         if length == 0:
             return None
-        #assert length == 1
+        assert length == 1
         return alist[0]
 
 class NewReferenceObject(BaseObject):
@@ -31,3 +33,10 @@ class NewReferenceObject(BaseObject):
     def __init__(self):
         if self.obj_id in [ None, '0' ]:
             self.obj_id = self.new_object_id()
+
+    def to_dict(self):
+        return save_object(self)
+
+    @classmethod
+    def from_dict(cls, song, data):
+        return load_object(cls, song, data)
