@@ -8,11 +8,20 @@
 
 from .base import NewReferenceObject
 from warpseq.playback.midi import open_port
+from warpseq.utils.serialization import Serializer
 
 class Device(NewReferenceObject):
 
     __slots__ = [ 'name', 'obj_id', '_midi_out' ]
 
+    SERIALIZER = Serializer(
+        values = ( 'name', ),
+        objects = (),
+        object_lists = (),
+        objects_2d = (),
+        custom=(),
+    )
+
     def __init__(self, name=None, obj_id=None):
         self.name = name
         self.obj_id = obj_id
@@ -23,16 +32,3 @@ class Device(NewReferenceObject):
         if self._midi_out is None:
             self._midi_out = open_port(self.name)
         return self._midi_out
-
-    def to_dict(self):
-        return dict(
-            obj_id = self.obj_id,
-            name = self.name
-        )
-
-    @classmethod
-    def from_dict(cls, song, data):
-        return Device(
-            obj_id = data['obj_id'],
-            name = data['name']
-        )
