@@ -9,7 +9,7 @@
 
 from .base import NewReferenceObject
 from .note import Note
-import functools
+from warpseq.utils.serialization import Serializer
 
 SCALE_TYPES = dict(
    major              = [ 1, 2, 3, 4, 5, 6, 7 ],
@@ -50,6 +50,14 @@ class Scale(NewReferenceObject):
 
     __slots__ = ['name', 'root', 'scale_type', 'slots', '_cached', '_cached_numbers', 'obj_id']
 
+    SERIALIZER = Serializer(
+        values = ( 'name', 'scale_type', 'slots' ),
+        objects = ( ),
+        object_lists = (),
+        objects_2d = (),
+        custom = ('root'),
+    )
+
     def __init__(self, name=None, root=None, scale_type=None, slots=None, obj_id=None):
         self.name = name
         self.root = root
@@ -63,33 +71,20 @@ class Scale(NewReferenceObject):
     def get_notes(self):
         return self._cached
 
+    def get_first(self):
+        return self._cached[0]
+
     def get_note_numbers(self):
         return self._cached_numbers
 
-    def to_dict(self):
-        data = dict(
-            obj_id = self.obj_id,
-            name = self.name,
-            scale_type = self.scale_type,
-            slots = self.slots
-        )
+    def save_root(self):
         if self.root:
-            data["root"] = [ self.root.name, self.root.octave ]
-        else:
-            data["root"] = None
-        return data
+            return [ self.root.name, self.root.octave ]
+        return None
 
-    @classmethod
-    def from_dict(cls, song, data):
-        s = Scale(
-            obj_id = data['obj_id'],
-            name = data['name'],
-            scale_type = data['scale_type'],
-            slots = data['slots']
-        )
-        if data['root']:
-            s.root = Note(name=data['root'][0], octave=data['root'][1])
-        return s
+    def load_root(song, data):
+        x = data['root']
+        return Note(name=x[0], octave=x[1])
 
     def _internal_generate(self, length=120):
 
@@ -119,7 +114,8 @@ class Scale(NewReferenceObject):
                octave_shift = octave_shift + 1
 
             try:
-                result = self.root.transpose(degrees=scale_data[index], octaves=octave_shift)
+                result = self.root.copy().transpose(degrees=scale_data[index], octaves=octave_shift)
+                #print("res=%s" % result)
             except IndexError:
                 return
 
