@@ -9,17 +9,13 @@
 # process deferred mod-expressions caused by late-binding intra-track
 # events.
 
-#import rtmidi as rtmidi
-
 from warpseq.api.callbacks import Callbacks
 from warpseq.api.exceptions import *
 from warpseq.model.registers import register_playing_note, unregister_playing_note
-from warpseq.notation.mod import ModExpression
-
 from warpseq.playback.midi import midi_note_on, midi_note_off, midi_cc
-
 from warpseq.model.chord import Chord
 from warpseq.model.event import Event, NOTE_OFF, NOTE_ON
+from warpseq.model.context import Context
 
 def _get_note_number(note, instrument):
 
@@ -36,6 +32,7 @@ def _get_note_number(note, instrument):
     nn = note2.note_number()
 
     if nn < 0 or nn > 127:
+        print("NOTE: %s" % note)
         raise InvalidNote()
 
     return nn
@@ -43,21 +40,20 @@ def _get_note_number(note, instrument):
 
 class RealtimeEngine(object):
 
-    __slots__ = ['song','track','clip','midi_out','midi_port','mod_expressions','callbacks','on_count','player','on_ct']
+    __slots__ = ['song','track','clip','midi_out','midi_port','mod_expressions','callbacks','on_count','player','context']
 
     def __init__(self, song=None, track=None, clip=None, player=None):
-
         self.song = song
         self.track = track
         self.clip = clip
-        self.on_ct = 0
-
-
-        self.mod_expressions = ModExpression(defer=True, track=self.track, song=song)
-
         self.player = player
         self.callbacks = Callbacks()
-
+        self.context = Context(
+            player = self.player,
+            clip = self.clip,
+            track = self.track,
+            song = self.song,
+        )
 
     def play(self, event):
 
@@ -67,37 +63,31 @@ class RealtimeEngine(object):
         # because the mod expression does throw away the note value and capture the value from
         # the guide track...
 
-        pattern = event.note.from_parser.pattern
+        pattern = event.note.from_context.pattern
+
         cb = pattern.mod_expressions_callback # this must be a generator
 
-        if event.type == NOTE_ON and (cb or event.note.flags['deferred'] == True):
+        if event.type == NOTE_ON and (cb or event.note.deferred == True):
 
             # we have to process deferred expressions twice because of the mod events
             # UNLESS we pair the off event.
 
-            self.mod_expressions.scale = event.scale
-            self.mod_expressions.pattern = event.note.from_parser.pattern
-            self.mod_expressions.scale = event.note.from_scale
+            self.context.scale = event.scale
+            self.context.pattern = event.from_context.pattern
 
-            exprs = event.note.flags['deferred_expressions']
+            exprs = event.note.deferred_expressions
             for expr in exprs:
-                value = self.mod_expressions.do(event.note, expr)
+                value = expr.evaluate(context, note)
                 if value is None:
                     return
                 event.note = value
 
-            #event.note.flags['deferred'] = False
-
             # run mod expressions callback if attached
             if cb is not None:
-                value = event.note
-
                 new_mod_expressions = next(cb)
                 for expr in new_mod_expressions:
-                    value = self.mod_expressions.do(event.note, expr)
-                    #if type(value) == Chord:
-                    #    print("NOTES=%s" % value.notes)
-                    if value is None:
+                    event.note = expr.evaluate(context, event.note)
+                    if event.note is None:
                         return
                     event.note = value
 
@@ -113,9 +103,8 @@ class RealtimeEngine(object):
             for x in event.note.notes:
                 evt = event.copy()
                 evt.note = x
-                evt.note.flags['deferred'] = False
+                evt.note.deferred = False
                 self.play(evt)
-
             return
 
         if not event.note:
@@ -125,45 +114,25 @@ class RealtimeEngine(object):
 
             # FIXME: assign note vs always redeferencing event.note
 
-
-
             velocity = event.note.velocity
             if velocity is None:
                 velocity = self.instrument.default_velocity
 
             register_playing_note(self.track, event.note)
 
-            for (control, value) in event.note.flags['cc'].items():
+            for (control, value) in event.note.ccs.items():
                 control = int(control)
-
-
                 for instrument in self.track.get_instruments_to_play():
-
-                    #print("SEND MIDI CC: %s/%s" % (control, value))
-
-                    # command = (MIDI_CONTROLLER_CHANGE & 0xf0) | (self.channel - 1 & 0xf)
-                    # self.midi_out.send_message([command, channel & 0x7f, value & 0x7f])
-
-                    #msg = mido.Message(MIDO_CONTROLLER_CHANGE, channel=instrument.channel-1, control=control, value=int(value))
-                    #print(msg)
-                    #instrument.get_midi_out().send(msg)
-
                     midi_cc(instrument.get_midi_out(), instrument.channel, control, int(value))
 
 
             if not (self.track.muted or event.note.muted):
-
-                #print("%s ON: %s" % (self.track.name, event.note))
-                self.on_ct = self.on_ct + 1
                 self.player.inject_off_event(event)
-
                 for instrument in self.track.get_instruments_to_play():
                     if not instrument.muted:
-
+                        #print("PLAY: %s / %s" % (instrument.name, event.note))
                         midi_note_on(instrument.get_midi_out(), instrument.channel, _get_note_number(event.note, instrument), velocity)
 
-                        #msg = mido.Message(MIDO_NOTE_ON, note=_get_note_number(event.note, instrument), velocity=velocity, channel=instrument.channel-1)
-                        #instrument.get_midi_out().send(msg)
 
         elif event.type == NOTE_OFF:
 
@@ -180,18 +149,12 @@ class RealtimeEngine(object):
             velocity = event.note.velocity
             if velocity is None:
                 velocity = self.instrument.default_velocity
-            note_number = event.on_event.note.note_number()
 
             unregister_playing_note(self.track, event.on_event.note)
 
             if self.track.muted:
                 return
-
-            #print("OFF (%s): %s" % (self.on_ct, event.on_event.note))
-            self.on_ct = self.on_ct - 1
-
             for instrument in self.track.get_instruments_to_play():
                 if not instrument.muted:
-
                     midi_note_off(instrument.get_midi_out(), instrument.channel,  _get_note_number(event.note, instrument), velocity)
 
